"""
Package: aspects
Author: Ernesto Posse
Description:
    This package implements basic Aspect-Oriented Programming features.
    
    An aspect class defines a "pointcut" (the set of "joinpoints" where the 
    aspect is to be applied, and "advice" methods "before" and "after" to be
    executed before (resp. after) the joinpoint.
    
    A user-defined aspect is defined as a subclass of core.Aspect.

    The pointcut for an aspect is specified by defining the following class 
    attributes in the user-defined subclass of core.Aspect:
        classes
        methods
    In a future version, the following will be supported as well:
        get
        set
    Each of these class attributes is a regular expression. Together they define
    the classes and methods where the aspect is to be applied.
    
    The advice for an aspect is specified by defining methods "before" and 
    "after" in the user-defined subclass of core.Aspec. The parameters of these
    methods are the context in which the advice is applied:
        
        before(self, klass, method, obj, *args, **kwargs)
    
        after(self, klass, method, obj, retval, exc_type, exc_val, traceback)

    To weave (apply) one or more aspects A1, A2, ..., An, one must first create a
    meta-class using the WeaverMetaClassFactory function, as follows:
        
        MyWeaverMetaClass = core.WeaverMetaClassFactory(A1(), A2(), ..., An())

    Note that the arguments passed to the factory are Aspect instances rather
    than the classes themselves. This is because this is because particular
    aspects may need specifica initializations.
    
    Then, one should apply this meta-class to all modules and classes which are
    to be affected by the aspects. The simplest way to do this is to assign this
    meta-class to the class's __metaclass__ attribute, or, preferably, to the 
    global (module) __metaclass__ variable:
        
        __metaclass__ = MyWeaverMetaClass
        
    This package also implements a couple of commonly used aspects:
        - A general Logger aspect for logging.
        - A general Memoizer aspect for memoization or caching of method results
        
    See the tests/aspects folder for examples. 
"""

from core import *

