"""
Implements heaps.
"""

__all__ = ['Heap']

from collections import MutableSequence


class Heap(object):
    
    def __init__(self, data=None, ascending=False):
        assert isinstance(data, MutableSequence)
        self.array = data
        self.ascending = ascending
        if self.ascending:
            self.cmp = lambda x, y: x < y
        else:
            self.cmp = lambda x, y: x > y
    
    def isempty(self):
        return len(self.array) == 0
        
    def __contains__(self, datum):
        return datum in self.array
    
    def __iter__(self):
        return iter(self.array)
        
    def parent(self, i):
        return int(i/2)
    
    def right(self, i):
        return i * 2 + 1
    
    def left(self, i):
        return i * 2 

    def heapify(self, i):
        while True:
            l = self.left(i)
            r = self.right(i)
            if l < len(self.array) and self.cmp(self.array[l], self.array[i]):
                selected = l
            else:
                selected = i
            if r < len(self.array) and self.cmp(self.array[r], self.array[selected]):
                selected = r
            if selected != i:
                temp = self.array[i]
                self.array[i] = self.array[selected]
                self.array[selected] = temp
                i = selected
            else:
                break
    
    def add(self, datum):
        self.array.append(datum)
        i = len(self.array) - 1
        while True:
            parent = self.parent(i)
            if parent >= 0 and self.cmp(self.array[i], self.array[parent]):
                temp = self.array[parent]
                self.array[parent] = self.array[i]
                self.array[i] = temp
                i = parent
            else:
                break
    
    def peek_first(self):
        return self.array[0]
    
    def remove_first(self):
        return self.delete(0)
    
    def delete(self, index):
        first = self.array[index]
        self.array[index] = self.array[-1]
        del self.array[-1]
        self.heapify(index)
        return first
    
    def get(self, index):
        return self.array[index]
    
    def index(self, datum):
        i = 0
        while i < len(self.array):
            if datum == self.array[i]:
                return i
            i += 1
        raise IndexError()
    
    def remove(self, datum):
        return self.delete(self.index(datum))
    
    def __repr__(self):
        return "%s" % repr(self.array)
        
            
