"""
Module: order
Author: Ernesto Posse
Description: 
  An implementation of linearlly ordered sets.
"""

from posets import Top, Bottom, Poset


class LinearOrder(Poset):
    
    def __init__(self, elements=None):
        if elements is None:
            self.elements = []
        self.elements = elements
        self.indices = dict(zip(elements, range(len(elements))))
        self.indices[Top()] = len(elements)
        self.indices[Bottom()] = -1
    
    def __getitem__(self, index):
        return self.elements[index]
    
    def __iter__(self):
        return iter(self.elements)
    
    def __len__(self):
        return len(self.elements)
    
    def cmp(self, x, y):
        return cmp(self.indices[x], self.indices[y])
    
    def cmp_linear(self, x, y):
        #~ return cmp(self.elements.index(x), self.elements.index(y))
        if x == y:
            return 0
        x_index = None
        y_index = None
        i = 0
        while i < len(self.elements):
            element = self.elements[i]
            if element == x:
                x_index = i
            elif element == y:
                y_index = i
            i += 1
        if x_index is None or y_index is None:
            raise ValueError('incomparable elements')
        return cmp(x_index, y_index)
    
    def index(self, x):
        return self.indices[x]
        

