"""
Module: infinity
Author: Ernesto Posse
Created on: Jul 6, 2007
Last modified: Aug 6, 2010

Description:
  This implements a class to represent infinity and perform arithmetic operations (e.g. num + inf = inf)
"""

import sys


class Undefined(Exception):
    pass
    
    
class Infinity:

    def __repr__(self):
        return "Infinity()"

    def __int__(self):
        return sys.maxint
    
    def __float__(self):
        return float(sys.maxint)
        
    # Comparisons
    
    def __gt__(self, other):
        t = type(other)
        return t is int or t is float or other is MINUS_INFINITY or isinstance(other, NegativeInfinity)
        
    def __ge__(self, other):
        t = type(other)
        return t is int or t is float or other is MINUS_INFINITY or isinstance(other, NegativeInfinity)

    def __lt__(self, other):
        return False

    def __le__(self, other):
        return False

    # Arithmetic operations
    
    def __add__(self, other):
        if other is MINUS_INFINITY or isinstance(other, NegativeInfinity):
            raise Undefined()
        return self

    def __sub__(self, other) :
        if other is INFINITY or isinstance(other, Infinity):
            raise Undefined()
        return self

    def __mul__(self, other):
        pass

    def __floordiv__(self, other):
        pass
    
    def __radd__(self, other):
        if other is MINUS_INFINITY or isinstance(other, NegativeInfinity):
            raise Undefined()
        return self

    def __rsub__(self, other) :
        if other is INFINITY or isinstance(other, Infinity):
            raise Undefined()
        return MINUS_INFINITY

    def __rmul__(self, other):
        pass

    def __rfloordiv__(self, other):
        pass
    
    def __div__(self, other):
        print "div",self,other
        if type(other) in [int, float]:
            return self
        else:
            raise Undefined()
    
    def __rdiv__(self, other):
        if type(other) in [int, float]:
            return 0
        else:
            raise Undefined()
    
    
class NegativeInfinity:
    
    def __repr__(self):
        return "NegativeInfinity()"

    def __int__(self):
        return -sys.maxint - 1
    
    def __float__(self):
        return float(-sys.maxint - 1)
        
    # Comparisons
    
    def __gt__(self, other):
        return False

    def __ge__(self, other):
        return False

    def __lt__(self, other):
        t = type(other)
        return t is int or t is float or other is INFINITY or isinstance(other, Infinity)
        
    def __le__(self, other):
        t = type(other)
        return t is int or t is float or other is INFINITY or isinstance(other, Infinity)
    
    # Arithmetic operations
    
    def __add__(self, other):
        if other is INFINITY or isinstance(other, Infinity):
            raise Undefined()
        return self

    def __sub__(self, other) :
        if other is MINUS_INFINITY or isinstance(other, NegativeInfinity):
            raise Undefined()
        return self

    def __mul__(self, other):
        pass

    def __floordiv__(self, other):
        pass
    
    def __radd__(self, other):
        if other is INFINITY or isinstance(other, Infinity):
            raise Undefined()
        return self

    def __rsub__(self, other) :
        if other is MINUS_INFINITY or isinstance(other, NegativeInfinity):
            raise Undefined()
        return INFINITY

    def __rmul__(self, other):
        pass

    def __rfloordiv__(self, other):
        pass
    
    

INFINITY = Infinity()
MINUS_INFINITY = NegativeInfinity()

def cmp_tests():
    n1 = Infinity()
    n2 = NegativeInfinity()

    print 1, 1.618 < n1
    print 2, 1.618 <= n1
    print 3, 1.618 > n1
    print 4, 1.618 >= n1
    print
    print 5, 1.618 < n2
    print 6, 1.618 <= n2
    print 7, 1.618 > n2
    print 8, 1.618 >= n2
    print
    print '---------'
    print 1, n1 < 1.618
    print 2, n1 <= 1.618
    print 3, n1 > 1.618
    print 4, n1 >= 1.618
    print
    print 5, n2 < 1.618
    print 6, n2 <= 1.618
    print 7, n2 > 1.618
    print 8, n2 >= 1.618
    print

