"""
Author: Ernesto Posse
Created on: Nov 4, 2010
Description: 
    This module contains an extension to the TestCase class from the standard 
    unittest module to support an 'assertDoesNotRaise' method.
"""


import unittest
import types

ITERABLE_TYPES = [tuple, list, dict, set, frozenset, types.GeneratorType]

class ExtTestCase(unittest.TestCase):
    
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self.value_printer = str
    
    def assertRaises(self, exn_type, msg, callable_obj, *args, **kwargs):
        if msg is None:
            msg = ""
        raised = False
        try:
            callable_obj(*args, **kwargs)
        except Exception, e:
            raised = True
            msg = "%s raised (%s); but %s not raised (%s)" % (e.__class__.__name__, e, exn_type.__name__, msg)
            self.assertEqual(type(e), exn_type, msg)
        finally:
            if not raised:
                unittest.TestCase.assertRaises(self, exn_type, callable_obj, *args, **kwargs)
        
    def assertDoesNotRaise(self, exn_types, msg, callable_obj, *args, **kwargs):
        if msg is None:
            msg = ""
        if type(exn_types) not in ITERABLE_TYPES and issubclass(exn_types, Exception):
            exn_types = [exn_types]
        try:
            callable_obj(*args, **kwargs)
        except Exception, e:
            for exn_type in exn_types:
                msg = "%s raised (%s); and %s was raised indeed (%s)" % (e.__class__.__name__, e, exn_type.__name__, msg)
                self.assertNotEqual(type(e), exn_type, msg)
   
    def assertValueOrException(self, expected_val_or_exn_type, msg, callable_obj, *args, **kwargs):
        if msg is None:
            msg = ""
        if type(expected_val_or_exn_type) in [types.TypeType, types.ClassType] \
        and issubclass(expected_val_or_exn_type, Exception):
            self.assertRaises(expected_val_or_exn_type, msg, callable_obj, *args, **kwargs)
        else:
            actual_value = callable_obj(*args, **kwargs)
            msg = "%s != %s; %s" % (self.value_printer(actual_value), self.value_printer(expected_val_or_exn_type), msg)
            self.assertEqual(actual_value, expected_val_or_exn_type, msg)


