"""
Test cases for common languages
"""


import libutilitaspy.testing as testing


class LangOpMapTestCase(testing.ExtTestCase):
        
    def test_interp(self):
        # Test that the target of an interpretation is the expected value or exception."""
        for i in self.all_expr:
            msg = "term %s: '%s'; expected value: '%s'" % (i, self.pretty_printer.pformat(self.all_expr[i]), self.value_printer(self.expected_val_or_exn[i]))
            if i in self.eval_context:
                self.assertValueOrException(self.expected_val_or_exn[i], msg, self.interpreter, self.all_expr[i], self.eval_context[i])
            else:
                self.assertValueOrException(self.expected_val_or_exn[i], msg, self.interpreter, self.all_expr[i])


class LangPrettyPrintingTestCase(testing.ExtTestCase):
    
    def test_pretty_print(self):
        # Test that the printed string is what is expected."""
        for i in self.all_expr:
            s = self.pretty_printer.pformat(self.all_expr[i]) #, force_parenth = ['conjunction'])
            self.assertEqual(s, self.expected_pretty_str[i])
            
    def test_ast_pretty_print(self):
        #"""Test that the printed AST is what is expected, and that evaluating 
        #the printed AST results in a structurally equal object to the original 
        #AST."""
        for i in self.all_expr:
            s = self.ast_pretty_printer.pformat(self.all_expr[i])
            self.assertEqual(s, self.expected_pretty_ast_str[i], "Expression #%s\n%s" % (i,s))
            self.assertEqual(self.all_expr[i], eval(s, self.globals, self.locals))


class LangOpMapEqualityTestCase(testing.ExtTestCase):
    
    def test_map_equality(self):
        #"""Test that the application of two maps to all given expressions is 
        #equal."""
        for i in self.all_expr:
            expr = self.all_expr[i]
            eval_context = self.eval_context[i]
            value1 = self.map1(expr, eval_context)
            value2 = self.map2(expr, eval_context)
            self.assertEquals(value1, value2, "%s: %s != %s" % (expr, self.value_printer(value1), self.value_printer(value2)))
        
        
class LangTestCase(LangPrettyPrintingTestCase, LangOpMapTestCase):
    
    pass

